#=============================================================
#              Python3 program for monitoring
#
#    WVC-600 inverter via RF-interface on 433.400 MHz
#                          based on
#    USB-serial-module and HC-12 data-transceiver-module
#
# Author: Jaap van den Berg  Date: 27-DEC-2021  Version: 0.4
#
#=============================================================

#==========================================================================================
#---- Begin Input-data ---- Begin Input-data ---- Begin Input-data ---- Begin Input-data ---- Begin Input-data
#==========================================================================================
# !!!Check before running program!!!
#-----------------------------------

# Select applicable operating-system serial communication port -check first!-
# Windows: check Windows "configuration" -> "devices" for correct COM-port ("COM1", "COM2", "COM3",...)
# Linux:   check with command "ls -l /dev | grep USB" for correct device (e.g. "/dev/ttyUSB0")
computer_COM_port="/dev/ttyUSB0"

# Linux-note: Due to access-rights for "/dev/ttyUSB..", run program with "sudo python3 <program-name>.py"

# First part of CSV-file-name (without .csv file-extension: will be added including date-stamp) for monitoring-data
#   (csv-file used in append-mode: adding new records to the already existing csv-file)
#   (if no path to folder is included, current program-folder will be used)
#   (!beware: \T, \t, \N, \n, \R, \r are escape-characters, then use \\T, \\n, etc. instead! (example: "c:\\New-folder..) )
csv_filename_firstpart = "/wvc600-archive/wvc600-monitoring-data"

# WVC600-inverter-Id (at the sticker on the front of the WVC-600 inverter)
wvc600_id = "12345678"

# Maximum monitoring-time in daylight-mode (in minutes):
maximum_monitoring_time = 1200

# Time (sample period) between the monitoring-samples in daylight-mode
# (in seconds -preferably no less than 30-):
time_between_samples = 30

# Number of days to run program (to prevent that the program will run forever...)
number_of_days = 400

#==============================================================================================
#---- End Input-data ---- End Input-data ---- End Input-data ---- End Input-data ---- End Input-data
#==============================================================================================

# Import of necessary Python-modules
import time
import serial
import csv
import sys

# Construction of command to be sent to the WVC-600-inverter
command_firstpart_hex = "f20000000000000000"
command_lastpart_hex = "65"
command_total_hex = command_firstpart_hex + wvc600_id + command_lastpart_hex
command_to_wvc600 = bytes.fromhex(command_total_hex)

# Run the complete program for a maximum number of days
for day in range(number_of_days):

# Start program in awaiting-daylight-mode

# Open serial communication port at 9600 bits/sec (default 8 data-bits, no parity)
   serial_comm_port = serial.Serial(port=computer_COM_port, baudrate=9600, timeout=1, write_timeout=1)

# Set response and power-output from wvc-600-inverter to zero
   wvc600_response = "0"
   Pout = 0

# As long as there is no power-output > 0 response from wvc-600-inverter, stay in
# awaiting-daylight-mode while-loop and check communication with WVC-600-inverter every 15 minutes
   while Pout == 0:
# Send command to wvc-600-inverter
      serial_comm_port.write(command_to_wvc600)
# Receive the 28 bytes response from wvc-600-inverter
      wvc600_response = serial_comm_port.read(28)
      if len(wvc600_response) == 28:
         wvc600_hex_output = wvc600_response.hex()
         Pout_hex = wvc600_hex_output[48:52]
         Pout = int(Pout_hex,16) / 10
      time.sleep(90)
# End of awaiting-daylight-mode while-loop


# Program now continues in daylight-mode

# Construction of the complete daily csv-file name (including date-stamp)
   from time import gmtime, strftime, localtime
   date_stamp=strftime("-%Y-%m-%d", localtime())
   file_name_extension = ".csv"
   csv_filename = csv_filename_firstpart + date_stamp + file_name_extension

# Activation of a new csv-file for a new day that will be used in
# append-mode (adding new records to the already existing csv-file)
# Check if csv_file already exists. If not, create it...
   try:
       csv_file=open(csv_filename, 'rt')
       csv_file.close()
       csv_file=open(csv_filename, 'at', newline='', encoding="utf-8")
       writer = csv.writer(csv_file, dialect='excel', delimiter=';', quoting=csv.QUOTE_NONNUMERIC)
   except IOError:
# Creation of csv-file if necessary
       csv_file=open(csv_filename, 'wt', newline='', encoding="utf-8")
       writer = csv.writer(csv_file, dialect='excel', delimiter=';', quoting=csv.QUOTE_NONNUMERIC)

# Wait another 3 minutes to guarantee adequate day-light-level for more stable operation of
# the wvc-600-inverter in daylight-mode
   time.sleep(180)

# Calculation of the maximum number of samples during daylight-mode based on
# maximum monitoring time and time between samples (sample period)
   maximum_number_of_samples = int(60 * maximum_monitoring_time / time_between_samples)

# Set the number of executed samples and the number of error-responses of WVC-600-inverter to zero
   number_of_executed_samples = 0
   number_of_error_responses = 0

# Start monitoring of WVC-600 inverter in the daylight-mode while-loop, as long as the WVC-600-inverter
# is producing no more than 3 error responses in a row (when the solar-panels end their power output) and
# the number of executed samples is less than the maximum number of samples
   while number_of_error_responses <= 2 and number_of_executed_samples <= maximum_number_of_samples:

# Send command to wvc-600-inverter
      serial_comm_port.write(command_to_wvc600)

# Receive the 28 bytes response from wvc-600-inverter
      wvc600_response_new = serial_comm_port.read(28)

# Check on valid response from WVC-600: if no valid response (length not equal to 28 bytes),
# keep the last response and increase number of error-responses
      if len(wvc600_response_new) == 28:
         wvc600_response = wvc600_response_new
         number_of_error_responses = 0
      else:
         number_of_error_responses = number_of_error_responses + 1

# Increase number of executed samples
      number_of_executed_samples = number_of_executed_samples + 1

# Convert response from bytes to hex
      wvc600_hex_output = wvc600_response.hex()

# Calculation of WVC-600-inverter measurement-data from hex-output

# Calculation of solar-panels Vdc-in
      Vdc_hex = wvc600_hex_output[28:32]
      Vdc = int(Vdc_hex,16) / 100

# Calculation of output-voltage to grid Vac-out
      Vac_hex = wvc600_hex_output[32:36]
      Vac = int(Vac_hex,16) / 100

# Calculation of solar-panels Idc-in
      Idc_hex = wvc600_hex_output[36:40]
      Idc = int(Idc_hex,16) / 100

# Calculation of total energy generated by WVC-600-inverter E-total
      Etot_hex = wvc600_hex_output[40:44]
      Etot = int(Etot_hex,16) / 100

# Calculation of output power to grid P-out
      Pout_hex = wvc600_hex_output[48:52]
      Pout = int(Pout_hex,16) / 10

# Calculation of temperature of WVC-600-inverter Temp
      Temp_hex = wvc600_hex_output[52:56]
      Temp = int(Temp_hex,16) / 100

# Construction of date-stamp:
      from time import gmtime, strftime, localtime
      date_stamp=strftime("%Y-%m-%d", localtime())

# Construction of time-stamp:
      from time import gmtime, strftime, localtime
      time_stamp=strftime("%H:%M:%S", localtime())

# Append calculated measurement data to csv-file:
      writer.writerow([date_stamp,time_stamp,Vdc,Idc,Vac,Pout,Temp,Etot])

# Wait for executing next sample
      time.sleep(time_between_samples)

# End of the day-mode while-loop (after more than 3 error responses in a row or exceeding the maximum number of samples)

# After one minute, close serial communication port and csv-file
   time.sleep(60)
   serial_comm_port.close()
   csv_file.close()

# Program now continues in night-mode

# Provide a night-time-delay (length in seconds, depending on the actual month-number) in which communication
# to the WVC-600-inverter is shut down in order to avoid needless RF-transmissions during the night

   def night_time_delay(i):
          switcher={
                   1:47700,
                   2:41400,
                   3:34200,
                   4:27000,
                   5:20700,
                   6:17100,
                   7:18900,
                   8:25200,
                   9:32400,
                   10:39600,
                   11:45900,
                   12:49500
                }
          return switcher.get(i,18000)
   from time import gmtime, strftime, localtime
   month_number=int(strftime("%m", localtime()))
   time.sleep(night_time_delay(month_number))

# After this nigh-time-delay, return to awaiting-daylight-mode


# Program will end anyway after the number_of_days of operation
